@@ -76,30 +76,34 @@ set_bin_path()
     export PATH="${unique_steam_runtime_paths}${PATH}"
 }
 
-host_library_paths=
+set_bin_path
 
 if [[ "${STEAM_RUNTIME_PREFER_HOST_LIBRARIES-}" == "0" ]]; then
     log "STEAM_RUNTIME_PREFER_HOST_LIBRARIES=0 is deprecated, and no longer has an effect."
 fi
 
-exit_status=0
-ldconfig_output=$(/sbin/ldconfig -XNv 2> /dev/null; exit $?) || exit_status=$?
-if [[ $exit_status != 0 ]]; then
-    log "Warning: An unexpected error occurred while executing \"/sbin/ldconfig -XNv\", the exit status was $exit_status"
-fi
-
-# Always prefer host libraries over non-pinned Runtime libraries.
-# (In older versions this was conditional, but if we don't do this,
-# it usually breaks Mesa drivers' dependencies.)
-while read -r line; do
-    # If line starts with a leading / and contains :, it's a new path prefix
-    if [[ "$line" =~ ^/.*: ]]
-    then
-        library_path_prefix=$(echo "$line" | cut -d: -f1)
-
-        host_library_paths=$host_library_paths$library_path_prefix:
+if host_library_paths="$(steam-runtime-identify-library-abi --ldconfig-paths --one-line --quiet)"; then
+    # Code below assumes a trailing ":"
+    host_library_paths="${host_library_paths}:"
+else
+    log "steam-runtime-identify-library-abi --ldconfig-paths failed, falling back to ldconfig"
+    host_library_paths=
+    exit_status=0
+    ldconfig_output=$(/sbin/ldconfig -XNv 2> /dev/null; exit $?) || exit_status=$?
+    if [[ $exit_status != 0 ]]; then
+        log "Warning: An unexpected error occurred while executing \"/sbin/ldconfig -XNv\", the exit status was $exit_status"
     fi
-done <<< "$ldconfig_output"
+
+    while read -r line; do
+        # If line starts with a leading / and contains :, it's a new path prefix
+        if [[ "$line" =~ ^/.*: ]]
+        then
+            library_path_prefix=$(echo "$line" | cut -d: -f1)
+
+            host_library_paths=$host_library_paths$library_path_prefix:
+        fi
+    done <<< "$ldconfig_output"
+fi
 
 host_library_paths="${LD_LIBRARY_PATH:+"${LD_LIBRARY_PATH}:"}$host_library_paths"
 
@@ -124,6 +128,24 @@ case "${STEAM_RUNTIME_USE_LIBCURL_SHIM-}" in
         ;;
 esac
 
+case "${STEAM_RUNTIME_PIN_32BIT_LIBDBUS-}" in
+    (1)
+        log "Warning: \$STEAM_RUNTIME_PIN_32BIT_LIBDBUS will be removed in a future version."
+        log "Warning: Report a bug at https://github.com/ValveSoftware/steam-runtime/issues if you need this option."
+        host_library_paths="$STEAM_RUNTIME/pinned_32bit_libdbus:$host_library_paths"
+        [ -d "$STEAM_RUNTIME/pinned_32bit_libdbus" ] || mkdir "$STEAM_RUNTIME/pinned_32bit_libdbus"
+        ln -fns ../lib/i386-linux-gnu/libdbus-1.so.3 "$STEAM_RUNTIME/pinned_32bit_libdbus/"
+        ;;
+    (0 | '')
+        ;;
+    (*)
+        log "Warning: \$STEAM_RUNTIME_PIN_32BIT_LIBDBUS should be 0, 1 or empty, not '$STEAM_RUNTIME_PIN_32BIT_LIBDBUS'"
+        ;;
+esac
+
+# Always prefer host libraries over non-pinned Runtime libraries.
+# (In older versions this was conditional, but if we don't do this,
+# it usually breaks Mesa drivers' dependencies.)
 steam_runtime_library_paths="$host_library_paths$STEAM_RUNTIME/lib/i386-linux-gnu:$STEAM_RUNTIME/usr/lib/i386-linux-gnu:$STEAM_RUNTIME/lib/x86_64-linux-gnu:$STEAM_RUNTIME/usr/lib/x86_64-linux-gnu:$STEAM_RUNTIME/lib:$STEAM_RUNTIME/usr/lib"
 
 if [[ -n "${STEAM_COMPAT_INSTALL_PATH-}" && -n "${STEAM_COMPAT_FLAGS-}" ]]; then
@@ -144,8 +166,6 @@ fi
 
 export LD_LIBRARY_PATH="$steam_runtime_library_paths"
 
-set_bin_path
-
 exec "$@"
 
 # vi: ts=4 sw=4 expandtab
