@@ -23,6 +23,7 @@ export TEXTDOMAIN=steam
 export TEXTDOMAINDIR=/usr/share/locale
 
 ARCHIVE_EXT=tar.xz
+log_opened=
 
 # figure out the absolute path to the script being run a bit
 # non-obvious, the ${0%/*} pulls the path out of $0, cd's into the
@@ -59,6 +60,7 @@ export SYSTEM_ZENITY="$(which zenity 2>/dev/null)"
 if [ -z ${SYSTEM_ZENITY} ]; then
 	export STEAM_ZENITY="zenity"
 else
+	# FIXME: host zenity may fail because of the scout LD_* edits and should be run under SYSTEM_LD_LIBRARY_PATH
 	export STEAM_ZENITY="${SYSTEM_ZENITY}"
 fi
 
@@ -81,7 +83,19 @@ function show_message()
 
 	log_e "$title: $*"
 
+	if [ "${XDG_CURRENT_DESKTOP:-other}" == "gamescope" ]; then
+		# any attempt to run zenity or xterm under gamescope will block with nothing displayed
+		return
+	fi
+
 	if [ -n "${STEAM_ZENITY}" ]; then
+
+		if [ "${STEAM_ZENITY}" = "${SYSTEM_ZENITY}" ]; then
+			# FIXME: we have to go back to system LD_ setup otherwise system zenity may not work
+			LD_LIBRARY_PATH_BAK="${LD_LIBRARY_PATH}"
+			export LD_LIBRARY_PATH="${SYSTEM_LD_LIBRARY_PATH}"
+		fi
+
 		if ! "${STEAM_ZENITY}" "$style" --text="$*" --width=450 2>/dev/null; then
 			# Save the prompt in a temporary file because it can have newlines in it
 			tmpfile="$(mktemp || echo "/tmp/steam_message.txt")"
@@ -90,12 +104,66 @@ function show_message()
 				(echo "$title:"; cat "$tmpfile"; echo -n 'Press enter to continue: '; read input) >&2
 			rm -f "$tmpfile"
 		fi
+
+		if [ "${STEAM_ZENITY}" = "${SYSTEM_ZENITY}" ]; then
+			export LD_LIBRARY_PATH="${LD_LIBRARY_PATH_BAK}"
+		fi
+
 	else
 		# Keep a trace on disk at least
 		echo -e "[$(date +'%Y-%m-%dT%H:%M:%S%z')]: $title: $*" >> /tmp/steam_startup_messages_$USER.txt
 	fi
 }
 
+function maybe_open_log()
+{
+	local srt="$1"
+	local data="$2"
+	local argv="$3"
+
+	case " $argv " in
+		(*\ -srt-logger-opened\ *)
+			log "Log already open"
+			return 0
+			;;
+	esac
+
+	if [ -n "$log_opened" ]; then
+		return 0
+	fi
+
+	if [ "${STEAM_RUNTIME_LOGGER-}" = "0" ]; then
+		# Interferres with vscode's gdb wrapping for instance
+		log "Logging to console-linux.txt disabled via STEAM_RUNTIME_LOGGER"
+		return 0
+	fi
+
+	if [ "x${DEBUGGER-}" != "x" ]; then
+		# Interferres with ncurses (cgdb etc.)
+		log "Setting up for debugging, not logging to console-linux.txt"
+		return 0
+	fi
+
+	local logger="${srt}/amd64/usr/bin/srt-logger"
+
+	if ! [ -x "$logger" ]; then
+		log "Couldn't find $logger, logging to console-linux.txt"
+		return 0
+	fi
+
+	mkdir -p "$data/${STEAM_CLIENT_LOG_FOLDER:-logs}"
+
+	if [ -z "${SRT_LOG_TERMINAL+set}" ]; then
+		if [ -t 2 ] && SRT_LOG_TERMINAL=$(tty); then
+			export SRT_LOG_TERMINAL
+		else
+			export SRT_LOG_TERMINAL=
+		fi
+	fi
+
+	exec > >( exec "$logger" --exec-fallback --filename=console-linux.txt -t steam ) 2>&1
+	log_opened=1
+}
 
 function distro_description()
 {
@@ -327,11 +395,11 @@ function extract_archive()
 function has_runtime_archive()
 {
 	# Make sure we have files to unpack
-	if [ ! -f "$STEAM_RUNTIME.$ARCHIVE_EXT.part0" ]; then
+	if [ ! -f "$STEAM_RUNTIME.$ARCHIVE_EXT" ]; then
 		return 1
 	fi
 
-	if [ ! -f "$STEAM_RUNTIME.checksum" ]; then
+	if [ ! -f "$STEAM_RUNTIME.$ARCHIVE_EXT.checksum" ]; then
 		return 1
 	fi
 
@@ -350,7 +418,7 @@ function unpack_runtime()
 	fi
 
 	# Make sure we haven't already unpacked them
-	if [ -f "$STEAM_RUNTIME/checksum" ] && [[ $(< "$STEAM_RUNTIME.checksum" ) = $(< "$STEAM_RUNTIME/checksum" ) ]] ; then
+	if [ -f "$STEAM_RUNTIME/checksum" ] && [[ $(< "$STEAM_RUNTIME.$ARCHIVE_EXT.checksum" ) = $(< "$STEAM_RUNTIME/checksum" ) ]] ; then
 		check_pins "$STEAM_RUNTIME"
 		return 0
 	fi
@@ -359,9 +427,8 @@ function unpack_runtime()
 	EXTRACT_TMP="$STEAM_RUNTIME.tmp"
 	rm -rf "$EXTRACT_TMP"
 	mkdir "$EXTRACT_TMP"
-	cat "$STEAM_RUNTIME.$ARCHIVE_EXT".part* >"$STEAM_RUNTIME.$ARCHIVE_EXT"
 	EXISTING_CHECKSUM="$(cd "$(dirname "$STEAM_RUNTIME")"; md5sum "$(basename "$STEAM_RUNTIME.$ARCHIVE_EXT")")"
-	EXPECTED_CHECKSUM="$(cat "$STEAM_RUNTIME.checksum")"
+	EXPECTED_CHECKSUM="$(cat "$STEAM_RUNTIME.$ARCHIVE_EXT.checksum")"
 	if [ "$EXISTING_CHECKSUM" != "$EXPECTED_CHECKSUM" ]; then
 		log $"Runtime checksum: $EXISTING_CHECKSUM, expected $EXPECTED_CHECKSUM"
 		return 2
@@ -381,7 +448,7 @@ function unpack_runtime()
 		return 5
 	fi
 	rm -rf "$EXTRACT_TMP"
-	if ! cp "$STEAM_RUNTIME.checksum" "$STEAM_RUNTIME/checksum"; then
+	if ! cp "$STEAM_RUNTIME.$ARCHIVE_EXT.checksum" "$STEAM_RUNTIME/checksum"; then
 		return 6
 	fi
 	# Unpacked a new runtime, pin any newer system libs with symlinks in a special dir
@@ -535,7 +602,7 @@ function steamos_arg()
 	return 1; # 1 == false in bash speak
 }
 
-#determine platform
+# determine platform
 UNAME=`uname`
 if [ "$UNAME" != "Linux" ]; then
    show_message --error "Unsupported Operating System"
@@ -571,10 +638,8 @@ STEAMSTARTING="$STEAMCONFIG/starting"
 
 # Was -steamos specified
 : "${STEAMOS:=}"
-if steamos_arg $@; then
+if steamos_arg "$@"; then
 	STEAMOS=1
-	# Disables progress bars and message boxes
-	STEAM_ZENITY=
 fi
 
 # See if this is the initial launch of Steam
@@ -721,6 +786,11 @@ if [ "${1-}" = "--run" ]; then
 	exit 255
 fi
 
+maybe_open_log "$STEAM_RUNTIME" "${STEAMDATALINK:-"$STEAMCONFIG/steam"}" "$*"
+if [ -n "$log_opened" ]; then
+	set -- -srt-logger-opened "$@"
+fi
+
 # prepend our lib path to LD_LIBRARY_PATH
 export LD_LIBRARY_PATH="$STEAMROOT/$PLATFORM:$STEAMROOT/$PLATFORM/panorama:${LD_LIBRARY_PATH-}"
 
