@@ -6,6 +6,9 @@
 # This is the Steam script that typically resides in /usr/bin
 # It will create the Steam bootstrap if necessary and then launch steam.
 
+# verbose
+#set -x
+
 set -e
 
 # Get the full name of this script
@@ -18,7 +21,7 @@ log () {
     echo "bin_steam.sh[$$]: $*" >&2 || :
 }
 
-export STEAMSCRIPT_VERSION=1.0.0.75
+export STEAMSCRIPT_VERSION=1.0.0.81
 
 # Set up domain for script localization
 export TEXTDOMAIN=steam
@@ -27,19 +30,25 @@ function show_message()
 {
 	style=$1
 	shift
-	if ! zenity "$style" --text="$*" 2>/dev/null; then
-		case "$style" in
-		--error)
-			title=$"Error"
-			;;
-		--warning)
-			title=$"Warning"
-			;;
-		*)
-			title=$"Note"
-			;;
-		esac
 
+	case "$style" in
+	--error)
+		title=$"Error"
+		;;
+	--warning)
+		title=$"Warning"
+		;;
+	*)
+		title=$"Note"
+		;;
+	esac
+
+	if [ "${XDG_CURRENT_DESKTOP}" == "gamescope" ]; then
+		log "$title: $*"
+		return
+	fi
+
+	if ! zenity "$style" --text="$*" 2>/dev/null; then
 		# Save the prompt in a temporary file because it can have newlines in it
 		tmpfile="$(mktemp || echo "/tmp/steam_message.txt")"
 		echo -e "$*" >"$tmpfile"
@@ -160,6 +169,22 @@ function check_bootstrap()
 	fi
 }
 
+function forward_command_line()
+{
+	if ! [ -p "$STEAMCONFIG/steam.pipe" ]; then
+		return 1
+	fi
+
+	local runtime="$STEAMCONFIG/root/ubuntu12_32/steam-runtime"
+	local remote="$runtime/amd64/usr/bin/steam-runtime-steam-remote"
+
+	if [ -x "$remote" ] && "$remote" "$@" 2>/dev/null; then
+		return 0
+	else
+		return 1
+	fi
+}
+
 # Don't allow running as root
 if [ "$(id -u)" == "0" ]; then
 	show_message --error $"Cannot run as root user"
@@ -169,6 +194,12 @@ fi
 # Look for the Steam data files
 setup_variables
 
+# If Steam is already running, try to forward the command-line to it.
+# If successful, there's nothing more to do.
+if forward_command_line "$@"; then
+	exit 0
+fi
+
 if ! check_bootstrap "$LAUNCHSTEAMDIR"; then
 	# See if we just need to recreate the data link
 	if check_bootstrap "$DEFAULTSTEAMDIR"; then
@@ -180,28 +211,8 @@ if ! check_bootstrap "$LAUNCHSTEAMDIR"; then
 	fi
 fi
 
-if [[ ! -L "$STEAMDATALINK" ]]; then
-	# We don't have the data link and we couldn't find an existing bootstrap, so just install the bootstrap
+if [[ ! -L "$STEAMDATALINK" ]] || ( ! check_bootstrap "$LAUNCHSTEAMDIR" ); then
 	install_bootstrap "$DEFAULTSTEAMDIR"
-elif ! check_bootstrap "$LAUNCHSTEAMDIR"; then
-	# We have the data link but we can't find the data, prompt the user
-	if zenity --question --text=$"Couldn't find Steam content, did you move it?" --ok-label=$"Search" --cancel-label=$"Reinstall"; then
-		while true; do
-			STEAMDIR="$(zenity --file-selection --directory)"
-			if [[ -z "$STEAMDIR" ]]; then
-				# User canceled
-				exit 0
-			fi
-			if check_bootstrap "$STEAMDIR"; then
-				repair_bootstrap "$STEAMDIR"
-				break;
-			else
-				show_message --error $"Please pick a valid Steam content directory"
-			fi
-		done
-	else
-		install_bootstrap "$DEFAULTSTEAMDIR"
-	fi
 fi
 
 if ! check_bootstrap "$LAUNCHSTEAMDIR"; then
@@ -209,8 +220,13 @@ if ! check_bootstrap "$LAUNCHSTEAMDIR"; then
 	exit 1
 fi
 
+# Leave a copy of the bootstrap tarball in ~/.steam so that Steam can
+# re-bootstrap itself if required
+if ! cmp -s "$LAUNCHSTEAMBOOTSTRAPFILE" "$LAUNCHSTEAMDIR/bootstrap.tar.xz"; then
+    cp "$LAUNCHSTEAMBOOTSTRAPFILE" "$LAUNCHSTEAMDIR/bootstrap.tar.xz"
+fi
+
 # go to the install directory and run the client
-cp "$LAUNCHSTEAMBOOTSTRAPFILE" "$LAUNCHSTEAMDIR/bootstrap.tar.xz"
 cd "$LAUNCHSTEAMDIR"
 
 exec "$LAUNCHSTEAMDIR/$STEAMBOOTSTRAP" "$@"
